@@ -15,7 +15,7 @@
 #include <string.h>
 #include <errno.h>
 
-__ID("@(#) $Id: fb.cc 1959 2008-05-09 20:32:50Z lyonel $");
+__ID("@(#) $Id: fb.cc 2069 2009-02-12 22:53:09Z lyonel $");
 
 #define FB_MODES_SHIFT  5                         /* 32 modes per framebuffer */
 #define FB_NUM_MINORS 256                         /* 256 Minors               */
@@ -167,7 +167,7 @@ struct fb_var_screeninfo
   u_int32_t reserved[6];                          /* Reserved for future compatibility */
 };
 
-static int lookup_dev(char *name)
+static int lookup_dev(const char *name)
 {
   FILE *f;
   int n;
@@ -192,11 +192,12 @@ static int lookup_dev(char *name)
 
 static int open_dev(dev_t dev)
 {
-  static char *paths[] =
+  static const char *paths[] =
   {
     "/var/run", "/dev", "/tmp", NULL
   };
-  char **p, fn[64];
+  char const **p;
+  char fn[64];
   int fd;
 
   for (p = paths; *p; p++)
