@@ -11,6 +11,7 @@
 #include "usb.h"
 #include "osutils.h"
 #include "heuristics.h"
+#include "options.h"
 #include <stdio.h>
 #include <map>
 #include <sys/types.h>
@@ -18,10 +19,13 @@
 #include <sys/ioctl.h>
 #include <errno.h>
 #include <fcntl.h>
+#include <string.h>
+#include <stdlib.h>
 #include <unistd.h>
 #include <dirent.h>
 
 #define PROCBUSUSBDEVICES "/proc/bus/usb/devices"
+#define SYSBUSUSBDEVICES "/sys/bus/usb/devices"
 #define USBID_PATH "/usr/share/lshw/usb.ids:/usr/local/share/usb.ids:/usr/share/usb.ids:/etc/usb.ids:/usr/share/hwdata/usb.ids:/usr/share/misc/usb.ids"
 
 #define USB_CLASS_PER_INTERFACE         0         /* for DeviceClass */
@@ -64,7 +68,7 @@ static map<u_int32_t,string> usbproducts;
 
 #define PRODID(x, y) ((x << 16) + y)
 
-__ID("@(#) $Id: usb.cc 1818 2007-08-05 21:11:59Z lyonel $");
+__ID("@(#) $Id: usb.cc 2069 2009-02-12 22:53:09Z lyonel $");
 
 static string usbhost(unsigned bus)
 {
@@ -274,12 +278,12 @@ static bool describeUSB(hwNode & device, unsigned vendor, unsigned prodid)
 {
   if(usbvendors.find(vendor)==usbvendors.end()) return false;
 
-  device.setVendor(usbvendors[vendor]);
+  device.setVendor(usbvendors[vendor]+(enabled("output:numeric")?" ["+tohex(vendor)+"]":""));
   device.addHint("usb.idVendor", vendor);
   device.addHint("usb.idProduct", prodid);
 
   if(usbproducts.find(PRODID(vendor, prodid))!=usbproducts.end())
-    device.setProduct(usbproducts[PRODID(vendor, prodid)]);
+    device.setProduct(usbproducts[PRODID(vendor, prodid)]+(enabled("output:numeric")?" ["+tohex(vendor)+":"+tohex(prodid)+"]":""));
 
   return true;
 }
@@ -298,7 +302,7 @@ static bool load_usbids(const string & name)
     char * buffer = NULL;
     size_t linelen;
     unsigned t = 0;
-    char description[100];
+    char *description = NULL;
 
     if(getline(&buffer, &linelen, usbids)>0)
     {
@@ -306,20 +310,26 @@ static bool load_usbids(const string & name)
         buffer[linelen-1] = '\0';                 // chop \n
       string line = string(buffer);
       free(buffer);
-      memset(description, 0, sizeof(description));
 
+      description = NULL;
+      t = 0;
       if(line.length()>0)
       {
         if(line[0] == '\t')                       // product id entry
         {
-          if(sscanf(line.c_str(), "%x %[ -z]", &t, description)>0)
+          line.erase(0, 1);
+          if(matches(line, "^[[:xdigit:]][[:xdigit:]][[:xdigit:]][[:xdigit:]]"))
+            t = strtol(line.c_str(), &description, 16);
+          if(description && (description != line.c_str()))
           {
             usbproducts[PRODID(vendorid,t)] = hw::strip(description);
           }
         }
         else                                      // vendor id entry
         {
-          if(sscanf(line.c_str(), "%x %[ -z]", &t, description)>0)
+          if(matches(line, "^[[:xdigit:]][[:xdigit:]][[:xdigit:]][[:xdigit:]]"))
+            t = strtol(line.c_str(), &description, 16);
+          if(description && (description != line.c_str()))
           {
             vendorid = t;
             usbvendors[t] = hw::strip(description);
@@ -342,14 +352,14 @@ bool scan_usb(hwNode & n)
   bool defined = false;
   unsigned int bus,lev,prnt,port,cnt,devnum,mxch;
   float speed;
-  char ver[10];
+  char ver[10+1];
   unsigned int cls, sub, prot, mxps, numcfgs;
   unsigned int vendor, prodid;
-  char rev[10];
+  char rev[10+1];
   unsigned numifs, cfgnum, atr;
-  char mxpwr[10];
+  char mxpwr[10+1];
   unsigned ifnum, alt, numeps;
-  char driver[80];
+  char driver[80+1];
 
   if (!exists(PROCBUSUSBDEVICES))
     return false;
@@ -363,13 +373,17 @@ bool scan_usb(hwNode & n)
   filenames.clear();
 
   usbdevices = fopen(PROCBUSUSBDEVICES, "r");
+#if 0
+  if(!usbdevices)
+    usbdevices = fopen(SYSBUSUSBDEVICES, "r");
+#endif
 
   while(!feof(usbdevices))
   {
     char * buffer = NULL;
     size_t linelen;
-    char strname[80];
-    char strval[80];
+    char strname[80+1];
+    char strval[80+1];
 
     if(getline(&buffer, &linelen, usbdevices)>0)
     {
@@ -435,7 +449,7 @@ bool scan_usb(hwNode & n)
           case 'P':
             vendor = prodid = 0;
             strcpy(rev, "");
-            if(sscanf(line.c_str(), "P: Vendor=%x ProdID=%x Rev=%s", &vendor, &prodid, rev)>0)
+            if(sscanf(line.c_str(), "P: Vendor=%x ProdID=%x Rev=%10s", &vendor, &prodid, rev)>0)
             {
               describeUSB(device, vendor, prodid);
               device.setVersion(hw::strip(rev));
@@ -444,12 +458,12 @@ bool scan_usb(hwNode & n)
           case 'S':
             memset(strname, 0, sizeof(strname));
             memset(strval, 0, sizeof(strval));
-            if(sscanf(line.c_str(), "S: %[^=]=%[ -z]", strname, strval)>0)
+            if(sscanf(line.c_str(), "S: %80[^=]=%80[ -z]", strname, strval)>0)
             {
               if(strcasecmp(strname, "Manufacturer")==0)
-                device.setVendor(hw::strip(strval));
+                device.setVendor(hw::strip(strval)+(enabled("output:numeric")?" ["+tohex(vendor)+"]":""));
               if(strcasecmp(strname, "Product")==0)
-                device.setProduct(hw::strip(strval));
+                device.setProduct(hw::strip(strval)+(enabled("output:numeric")?" ["+tohex(vendor)+":"+tohex(prodid)+"]":""));
               if(strcasecmp(strname, "SerialNumber")==0)
                 device.setSerial(hw::strip(strval));
             }
@@ -459,13 +473,14 @@ bool scan_usb(hwNode & n)
             strcpy(mxpwr, "");
             if(sscanf(line.c_str(), "C:* #Ifs=%u Cfg#=%u Atr=%x MxPwr=%s", &numifs, &cfgnum, &atr, mxpwr)>0)
             {
-              device.setConfig("maxpower", mxpwr);
+              if(strcmp("0mA", mxpwr)!=0)
+                device.setConfig("maxpower", mxpwr);
             }
             break;
           case 'I':
             ifnum = alt = numeps = cls = sub = prot = 0;
             memset(driver, 0, sizeof(driver));
-            if((sscanf(line.c_str(), "I: If#=%u Alt=%u #EPs=%u Cls=%x(%*5c) Sub=%x Prot=%x Driver=%[ -z]", &ifnum, &alt, &numeps, &cls, &sub, &prot, driver)>0) && (cfgnum>0))
+            if(((sscanf(line.c_str(), "I:* If#=%u Alt=%u #EPs=%u Cls=%x(%*5c) Sub=%x Prot=%x Driver=%80[ -z]", &ifnum, &alt, &numeps, &cls, &sub, &prot, driver)>0) && (cfgnum>0)) || ((sscanf(line.c_str(), "I: If#=%u Alt=%u #EPs=%u Cls=%x(%*5c) Sub=%x Prot=%x Driver=%80[ -z]", &ifnum, &alt, &numeps, &cls, &sub, &prot, driver)>0) && (cfgnum>0)))
             {
               setUSBClass(device, cls, sub, prot);
               if((strlen(driver)!=0) && (strcasecmp("(none)", driver)!=0))
